@@ -28,14 +28,14 @@ void config_and_update (gpointer data, guint callback_action, GtkWidget *w);
 void menu_quit (gpointer data, guint callback_action, GtkWidget *w);
 
 static GtkItemFactoryEntry menu_items[] = {
-    { "/_New Tab",        "<Ctrl><Shift>T", add_tab_menu_call,     0, "<Item>"                             },
-    { "/_Close Tab",      NULL,      close_tab,             0, "<Item>"                             },
-    { "/sep1",            NULL,      NULL,                  0, "<Separator>"                        },
+    { "/_New Tab", "<Ctrl><Shift>T", add_tab_menu_call,     0, "<Item>",      NULL                  },
+    { "/_Close Tab",      NULL,      close_tab,             0, "<Item>",      NULL                  },
+    { "/sep1",            NULL,      NULL,                  0, "<Separator>", NULL                  },
     { "/_Copy",           NULL,      copy,                  0, "<StockItem>", GTK_STOCK_COPY        },
     { "/_Paste",          NULL,      paste,                 0, "<StockItem>", GTK_STOCK_PASTE       },
-    { "/sep1",            NULL,      NULL,                  0, "<Separator>"                        },
+    { "/sep1",            NULL,      NULL,                  0, "<Separator>", NULL                  },
     { "/_Preferences...", NULL,      config_and_update,     0, "<StockItem>", GTK_STOCK_PREFERENCES },
-    { "/sep1",            NULL,      NULL,                  0, "<Separator>"                        },
+    { "/sep1",            NULL,      NULL,                  0, "<Separator>", NULL                  },
     { "/_Quit",         "<Ctrl>Q",   menu_quit,             0, "<StockItem>", GTK_STOCK_QUIT        }
 };
 
@@ -75,7 +75,7 @@ void free_and_remove (tilda_window *tw)
     puts("free_and_remove");
 #endif
 
-    int i;
+    guint i;
 
     remove (tw->lock_file);
 
@@ -85,6 +85,114 @@ void free_and_remove (tilda_window *tw)
     g_list_free (tw->terms);
 }
 
+void goto_tab (tilda_window *tw, guint i)
+{
+#ifdef DEBUG
+    puts("goto_tab");
+#endif
+
+    gtk_notebook_set_current_page (GTK_NOTEBOOK (tw->notebook), i);
+}
+
+void goto_tab_1 (tilda_window *tw)
+{
+#ifdef DEBUG
+    puts("goto_tab 1");
+#endif
+
+    if (g_list_length (tw->terms) > 1)
+        goto_tab (tw, g_list_length (tw->terms)-1);
+}
+
+void goto_tab_2 (tilda_window *tw)
+{
+#ifdef DEBUG
+    puts("goto_tab 2");
+#endif
+
+    if (g_list_length (tw->terms) > 1)
+        goto_tab (tw, g_list_length (tw->terms)-2);
+}
+
+void goto_tab_3 (tilda_window *tw)
+{
+#ifdef DEBUG
+    puts("goto_tab 3");
+#endif
+
+    if (g_list_length (tw->terms) > 2)
+        goto_tab (tw, g_list_length (tw->terms)-3);
+}
+
+void goto_tab_4 (tilda_window *tw)
+{
+#ifdef DEBUG
+    puts("goto_tab 4");
+#endif
+
+    if (g_list_length (tw->terms) > 3)
+        goto_tab (tw, g_list_length (tw->terms)-4);
+}
+
+void goto_tab_5 (tilda_window *tw)
+{
+#ifdef DEBUG
+    puts("goto_tab 5");
+#endif
+
+    if (g_list_length (tw->terms) > 4)
+        goto_tab (tw, g_list_length (tw->terms)-5);
+}
+
+void goto_tab_6 (tilda_window *tw)
+{
+#ifdef DEBUG
+    puts("goto_tab 6");
+#endif
+
+    if (g_list_length (tw->terms) > 5)
+        goto_tab (tw, g_list_length (tw->terms)-6);
+}
+
+void goto_tab_7 (tilda_window *tw)
+{
+#ifdef DEBUG
+    puts("goto_tab 7");
+#endif
+
+    if (g_list_length (tw->terms) > 6)
+        goto_tab (tw, g_list_length (tw->terms)-7);
+}
+
+void goto_tab_8 (tilda_window *tw)
+{
+#ifdef DEBUG
+    puts("goto_tab 8");
+#endif
+
+    if (g_list_length (tw->terms) > 7)
+        goto_tab (tw, g_list_length (tw->terms)-8);
+}
+
+void goto_tab_9 (tilda_window *tw)
+{
+#ifdef DEBUG
+    puts("goto_tab 9");
+#endif
+
+    if (g_list_length (tw->terms) > 8)    
+        goto_tab (tw, g_list_length (tw->terms)-9);
+}
+
+void goto_tab_10 (tilda_window *tw)
+{
+#ifdef DEBUG
+    puts("goto_tab 10");
+#endif
+
+    if (g_list_length (tw->terms) > 9)
+        goto_tab (tw, g_list_length (tw->terms)-10);
+}
 
 void next_tab (tilda_window *tw)
 {
@@ -112,6 +220,7 @@ void clean_up (tilda_window *tw)
 
     free_and_remove (tw);
     gtk_main_quit ();
+    exit (0);
 }
 
 void clean_up_no_main (tilda_window *tw)
@@ -123,6 +232,30 @@ void clean_up_no_main (tilda_window *tw)
     free_and_remove (tw);
 }
 
+void start_program(tilda_collect *collect)
+{
+    int argc;
+    char **argv;
+
+    if (g_shell_parse_argv (cfg_getstr (collect->tw->tc, "command"), &argc, &argv, NULL))
+    {
+        vte_terminal_fork_command (VTE_TERMINAL(collect->tt->vte_term),
+            argv[0], argv, NULL,
+            cfg_getstr (collect->tw->tc, "working_dir"),
+            TRUE, TRUE, TRUE);
+
+        g_strfreev (argv);
+    }
+    else
+    {
+        const gchar *command = g_getenv ("SHELL");
+        vte_terminal_fork_command (VTE_TERMINAL(collect->tt->vte_term),
+            command, NULL, NULL,
+            cfg_getstr (collect->tw->tc, "working_dir"),
+            TRUE, TRUE, TRUE);
+    }
+}
+
 void close_tab_on_exit (GtkWidget *widget, gpointer data)
 {
 #ifdef DEBUG
@@ -138,13 +271,12 @@ void close_tab_on_exit (GtkWidget *widget, gpointer data)
             case 2:
                 close_tab (data, 0, widget);
                 break;
-            case 1:
-                break;
             case 0:
-                vte_terminal_fork_command (VTE_TERMINAL(collect->tt->vte_term),
-                    cfg_getstr (collect->tw->tc, "command"), NULL, NULL,
-                    cfg_getstr (collect->tw->tc, "working_dir"),
-                    TRUE, TRUE, TRUE);
+                break;
+            case 1:
+                vte_terminal_feed (VTE_TERMINAL(collect->tt->vte_term), "\r\n\r\n", 4);
+
+                start_program (collect);
                 break;
             default:
                 break;
