@@ -49,6 +49,8 @@ GtkWidget *combo_backspace, *combo_delete;
 GtkWidget *button_font;
 GtkWidget *combo_scroll_pos, *check_scroll_on_keystroke;
 GtkWidget *check_animation;
+GtkWidget *label_tab_orientation, *combo_tab_orientation;
+GtkWidget *slider_height, *slider_width;
 
 gboolean in_main = FALSE;
 
@@ -257,6 +259,50 @@ void image_select (GtkWidget *widget, GtkWidget *label_image)
     }
 }
 
+int percentage_height (int current_height) 
+{
+    return (int) (((float) current_height) / ((float) display_height) * 100);
+}
+
+int percentage_width (int current_width) 
+{
+    return (int) (((float) current_width) / ((float) display_width) * 100);
+}
+
+void slider_height_changed ()
+{
+    gchar smh[6];
+    gint mh = (gint) gtk_range_get_value (GTK_RANGE (slider_height)) * (display_height) / 100;    
+
+    sprintf (smh, "%d", mh);
+
+    gtk_spin_button_update (GTK_SPIN_BUTTON (entry_height));
+    
+    gtk_entry_set_text (GTK_ENTRY (entry_height), smh);
+}
+
+void slider_width_changed ()
+{
+    gchar smw[6];
+    gint mw = (gint) gtk_range_get_value (GTK_RANGE (slider_width)) * (display_width) / 100;
+
+    sprintf (smw, "%d", mw);
+
+    gtk_spin_button_update (GTK_SPIN_BUTTON (entry_width));
+
+    gtk_entry_set_text (GTK_ENTRY (entry_width), smw);
+}
+
+void entry_height_changed ()
+{
+    gtk_range_set_value (GTK_RANGE (slider_height), percentage_height (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (entry_height))));
+}
+
+void entry_width_changed ()
+{
+    gtk_range_set_value (GTK_RANGE (slider_width), percentage_width (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (entry_width))));
+}
+
 GtkWidget* appearance (tilda_window *tw, tilda_term *tt)
 {
 #ifdef DEBUG
@@ -266,40 +312,69 @@ GtkWidget* appearance (tilda_window *tw, tilda_term *tt)
     GtkWidget *table;
     GtkWidget *label_image;
     GtkWidget *label_opacity;
+    GtkWidget *label_slider_height, *label_slider_width;
     GtkWidget *label_height, *label_width;
     GtkWidget *label_x_pos, *label_y_pos;
     GtkWidget *label_slide_sleep_usec;
  
-    char s_max_height[6], s_max_width[6];
-    char s_x_pos[6], s_y_pos[6];
+    gchar s_max_height[6], s_max_width[6];
+    gchar s_x_pos[6], s_y_pos[6];
 
-    table = gtk_table_new (3, 9, FALSE);
+    table = gtk_table_new (3, 12, FALSE);
     label_height = gtk_label_new ("Height in Pixels:");
     label_width = gtk_label_new ("Width in Pixels:");
     label_x_pos = gtk_label_new ("X Pixel Postion to Start:");
     label_y_pos = gtk_label_new ("Y Pixel Postion to Start:");
     label_opacity = gtk_label_new ("Level of Transparency:");
     label_image = gtk_label_new ("Background Image:");
+    label_slider_height = gtk_label_new ("Percentage of Screen Height:");
+    label_slider_width = gtk_label_new ("Percentage of Screen Width:");
 
-    entry_height = gtk_entry_new ();
-    entry_width = gtk_entry_new ();
+    entry_height = gtk_spin_button_new_with_range (0, MAX_INT, 1);
+    entry_width = gtk_spin_button_new_with_range (0, MAX_INT, 1);;
     entry_x_pos = gtk_entry_new ();
     entry_y_pos = gtk_entry_new ();
     slider_opacity = gtk_hscale_new_with_range (0, 100, 1);
+    slider_height = gtk_hscale_new_with_range (0, 100, 1);
+    slider_width = gtk_hscale_new_with_range (0, 100, 1);
 
     sprintf (s_max_height, "%d", cfg_getint (tw->tc, "max_height"));
     sprintf (s_max_width, "%d",  cfg_getint (tw->tc, "max_width"));
+    
+    gtk_range_set_value (GTK_RANGE (slider_height), percentage_height (cfg_getint (tw->tc, "max_height")));
+    gtk_range_set_value (GTK_RANGE (slider_width), percentage_width (cfg_getint (tw->tc, "max_width")));
+    
+    g_signal_connect_swapped (G_OBJECT (slider_height), "change-value",
+        G_CALLBACK (slider_height_changed), NULL);
+
+    g_signal_connect_swapped (G_OBJECT (slider_width), "change-value",
+        G_CALLBACK (slider_width_changed), NULL);        
+    
     sprintf (s_x_pos, "%d",      cfg_getint (tw->tc, "x_pos"));
     sprintf (s_y_pos, "%d",      cfg_getint (tw->tc, "y_pos"));
-    gtk_entry_set_text (GTK_ENTRY (entry_height), s_max_height);
-    gtk_entry_set_text (GTK_ENTRY (entry_width), s_max_width);
+    gtk_spin_button_set_value (GTK_SPIN_BUTTON (entry_height), cfg_getint (tw->tc, "max_height"));
+    gtk_spin_button_set_value (GTK_SPIN_BUTTON (entry_width), cfg_getint (tw->tc, "max_width"));
     gtk_entry_set_text (GTK_ENTRY (entry_x_pos), s_x_pos);
     gtk_entry_set_text (GTK_ENTRY (entry_y_pos), s_y_pos);
     gtk_range_set_value (GTK_RANGE (slider_opacity), cfg_getint (tw->tc, "transparency"));
+
+    g_signal_connect_swapped (G_OBJECT (entry_height), "value-changed",
+        G_CALLBACK (entry_height_changed), NULL);  
     
-    label_slide_sleep_usec = gtk_label_new ("Animated Delay (in usecs):");
+    g_signal_connect_swapped (G_OBJECT (entry_width), "value-changed",
+        G_CALLBACK (entry_width_changed), NULL);  
+    
+    label_slide_sleep_usec = gtk_label_new ("Animation Delay (in usecs):");
     spin_slide_sleep_usec = gtk_spin_button_new_with_range (0, MAX_INT, 1);
     gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin_slide_sleep_usec), cfg_getint (tw->tc, "slide_sleep_usec"));
+
+    label_tab_orientation = gtk_label_new("Orientation:");
+    combo_tab_orientation = gtk_combo_box_new_text    ();
+    gtk_combo_box_prepend_text ((GtkComboBox *)combo_tab_orientation, "RIGHT");
+    gtk_combo_box_prepend_text ((GtkComboBox *)combo_tab_orientation, "LEFT");
+    gtk_combo_box_prepend_text ((GtkComboBox *)combo_tab_orientation, "BOTTOM");
+    gtk_combo_box_prepend_text ((GtkComboBox *)combo_tab_orientation, "TOP");
+    gtk_combo_box_set_active ((GtkComboBox *)combo_tab_orientation, cfg_getint (tw->tc, "animation_orientation"));
     
     check_use_image = gtk_check_button_new_with_label ("Use Image for Background");
     check_animation = gtk_check_button_new_with_label ("Animated Pulldown");
@@ -338,31 +413,43 @@ GtkWidget* appearance (tilda_window *tw, tilda_term *tt)
     gtk_table_attach (GTK_TABLE (table), label_height, 0, 1, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
     gtk_table_attach (GTK_TABLE (table), entry_height, 1, 3, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
 
-    gtk_table_attach (GTK_TABLE (table), label_width, 0, 1, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
-    gtk_table_attach (GTK_TABLE (table), entry_width, 1, 3, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
+    gtk_table_attach (GTK_TABLE (table), label_slider_height,  0, 1, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
+    gtk_table_attach (GTK_TABLE (table), slider_height, 1, 3, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
+    
+    gtk_table_attach (GTK_TABLE (table), label_width, 0, 1, 2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
+    gtk_table_attach (GTK_TABLE (table), entry_width, 1, 3, 2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
 
-    gtk_table_attach (GTK_TABLE (table), label_x_pos, 0, 1, 2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
-    gtk_table_attach (GTK_TABLE (table), entry_x_pos, 1, 3, 2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
+    gtk_table_attach (GTK_TABLE (table), label_slider_width,  0, 1, 3, 4, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
+    gtk_table_attach (GTK_TABLE (table), slider_width, 1, 3, 3, 4, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
 
-    gtk_table_attach (GTK_TABLE (table), label_y_pos, 0, 1, 3, 4, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
-    gtk_table_attach (GTK_TABLE (table), entry_y_pos, 1, 3, 3, 4, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
+    gtk_table_attach (GTK_TABLE (table), label_x_pos, 0, 1, 4, 5, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
+    gtk_table_attach (GTK_TABLE (table), entry_x_pos, 1, 3, 4, 5, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
 
-    gtk_table_attach (GTK_TABLE (table), label_opacity,  0, 1, 4, 5, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
-    gtk_table_attach (GTK_TABLE (table), slider_opacity, 1, 3, 4, 5, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
+    gtk_table_attach (GTK_TABLE (table), label_y_pos, 0, 1, 5, 6, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
+    gtk_table_attach (GTK_TABLE (table), entry_y_pos, 1, 3, 5, 6, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
 
-    gtk_table_attach (GTK_TABLE (table), check_animation, 0, 1, 5, 6, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
-    gtk_table_attach (GTK_TABLE (table), label_slide_sleep_usec, 1, 2, 5, 6, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
-    gtk_table_attach (GTK_TABLE (table), spin_slide_sleep_usec,  2, 3, 5, 6, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
+    gtk_table_attach (GTK_TABLE (table), label_opacity,  0, 1, 6, 7, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
+    gtk_table_attach (GTK_TABLE (table), slider_opacity, 1, 3, 6, 7, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
 
-    gtk_table_attach (GTK_TABLE (table), check_use_image, 0, 3, 6, 7, GTK_EXPAND | GTK_FILL,GTK_FILL, 3, 3);
+    gtk_table_attach (GTK_TABLE (table), check_animation, 0, 1, 7, 8, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
+    gtk_table_attach (GTK_TABLE (table), label_slide_sleep_usec, 1, 2, 7, 8, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
+    gtk_table_attach (GTK_TABLE (table), spin_slide_sleep_usec,  2, 3, 7, 8, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
+    gtk_table_attach (GTK_TABLE (table), label_tab_orientation, 1, 2, 9, 10, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
+    gtk_table_attach (GTK_TABLE (table), combo_tab_orientation,  2, 3, 9, 10, GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
 
-    gtk_table_attach (GTK_TABLE (table), label_image,  0, 1, 7, 8, GTK_EXPAND | GTK_FILL,GTK_FILL, 3, 3);
-    gtk_table_attach (GTK_TABLE (table), button_image, 1, 3, 7, 8, GTK_EXPAND | GTK_FILL,GTK_FILL, 3, 3);
+    gtk_table_attach (GTK_TABLE (table), check_use_image, 0, 3, 10, 11, GTK_EXPAND | GTK_FILL,GTK_FILL, 3, 3);
+
+    gtk_table_attach (GTK_TABLE (table), label_image,  0, 1, 11, 12, GTK_EXPAND | GTK_FILL,GTK_FILL, 3, 3);
+    gtk_table_attach (GTK_TABLE (table), button_image, 1, 3, 11, 12, GTK_EXPAND | GTK_FILL,GTK_FILL, 3, 3);
 
     gtk_widget_show (entry_height);
     gtk_widget_show (label_height);
+    gtk_widget_show (label_slider_height);
+    gtk_widget_show (slider_height);
     gtk_widget_show (entry_width);
     gtk_widget_show (label_width);
+    gtk_widget_show (label_slider_width);
+    gtk_widget_show (slider_width);
     gtk_widget_show (label_opacity);
     gtk_widget_show (slider_opacity);
     gtk_widget_show (entry_x_pos);
@@ -375,6 +462,8 @@ GtkWidget* appearance (tilda_window *tw, tilda_term *tt)
     gtk_widget_show (check_use_image);
     gtk_widget_show (label_image);
     gtk_widget_show (button_image);
+    gtk_widget_show (label_tab_orientation);
+    gtk_widget_show (combo_tab_orientation);
 
     return table;
 }
@@ -653,7 +742,7 @@ void apply_settings (tilda_window *tw)
     FILE *fp;
     gchar *tmp_str;
     tilda_term *tt;
-    gint i;
+    guint i;
 
     cfg_setstr (tw->tc, "key", gtk_entry_get_text (GTK_ENTRY (entry_key)));
     cfg_setstr (tw->tc, "font", gtk_font_button_get_font_name (GTK_FONT_BUTTON (button_font)));
@@ -664,8 +753,8 @@ void apply_settings (tilda_window *tw)
     old_max_height = cfg_getint (tw->tc, "max_height");
     old_max_width =  cfg_getint (tw->tc, "max_width");
 
-    cfg_setint (tw->tc, "max_height", atoi (gtk_entry_get_text (GTK_ENTRY (entry_height))));
-    cfg_setint (tw->tc, "max_width", atoi (gtk_entry_get_text (GTK_ENTRY (entry_width))));
+    cfg_setint (tw->tc, "max_height", gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (entry_height)));
+    cfg_setint (tw->tc, "max_width", gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (entry_width)));
     cfg_setint (tw->tc, "lines", gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin_scrollback)));
     cfg_setint (tw->tc, "transparency", gtk_range_get_value (GTK_RANGE (slider_opacity)));
     cfg_setint (tw->tc, "x_pos", atoi (gtk_entry_get_text (GTK_ENTRY (entry_x_pos))));
@@ -679,6 +768,7 @@ void apply_settings (tilda_window *tw)
     cfg_setint (tw->tc, "scheme", gtk_combo_box_get_active ((GtkComboBox *) combo_schemes));
     cfg_setint (tw->tc, "scrollbar_pos", gtk_combo_box_get_active ((GtkComboBox *) combo_scroll_pos));
     cfg_setint (tw->tc, "slide_sleep_usec", gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin_slide_sleep_usec)));
+    cfg_setint (tw->tc, "animation_orientation", gtk_combo_box_get_active ((GtkComboBox *) combo_tab_orientation));
 
     gtk_color_button_get_color ((GtkColorButton *) text_color, &gdk_text_color);
     gtk_color_button_get_color ((GtkColorButton *) back_color, &gdk_back_color);
@@ -730,23 +820,16 @@ gint ok (tilda_window *tw)
 
     apply_settings (tw);
     exit_status = 0;
-    gtk_widget_destroy (wizard_window);
 
     if (in_main)
+    {
         gtk_main_quit();
+        in_main = FALSE;
+    }
 
     return (TRUE);
 }
 
-void apply (tilda_window *tw)
-{
-#ifdef DEBUG
-    puts("apply");
-#endif
-
-    apply_settings (tw);
-}
-
 gint exit_app (GtkWidget *widget, gpointer data)
 {
 #ifdef DEBUG
@@ -763,6 +846,16 @@ gint exit_app (GtkWidget *widget, gpointer data)
     return (FALSE);
 }
 
+static void wizard_window_response_cb(GtkDialog* wizard_window, 
+                                      int response, 
+                                      tilda_window *tw)
+{
+    if (response == GTK_RESPONSE_OK) {
+        ok (tw);
+    } 
+    
+}
+
 int wizard (int argc, char **argv, tilda_window *tw, tilda_term *tt)
 {
 #ifdef DEBUG
@@ -773,6 +866,7 @@ int wizard (int argc, char **argv, tilda_window *tw, tilda_term *tt)
     GtkWidget *table;
     GtkWidget *notebook;
     GtkWidget *label;
+    
     GtkWidget *table2;
     GtkWidget *image;
     GdkPixmap *image_pix;
@@ -782,6 +876,9 @@ int wizard (int argc, char **argv, tilda_window *tw, tilda_term *tt)
     gchar title[20];
     gchar *tabs[] = {"General", "Title and Command", "Appearance", "Colors", "Scrolling", "Compatibility", "Keybindings"};
 
+    FILE *fp;
+    gint i;
+
     GtkWidget* (*contents[7])(tilda_window *, tilda_term *);
 
     contents[0] = general;
@@ -792,9 +889,6 @@ int wizard (int argc, char **argv, tilda_window *tw, tilda_term *tt)
     contents[5] = compatibility;
     contents[6] = keybindings;
 
-    FILE *fp;
-    gint i;
-
     if (argv != NULL)
         argv0 = argv[0];
 
@@ -806,26 +900,37 @@ int wizard (int argc, char **argv, tilda_window *tw, tilda_term *tt)
         gtk_init (&argc, &argv);
     }
 
-    wizard_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_resizable (GTK_WINDOW(wizard_window), FALSE);
-    gtk_window_set_position(GTK_WINDOW(wizard_window), GTK_WIN_POS_CENTER);
-    gtk_widget_realize (wizard_window);
+    wizard_window = gtk_dialog_new_with_buttons (
+      "Configure Tilda",
+      NULL,
+      0,
+      GTK_STOCK_QUIT,
+      GTK_RESPONSE_CANCEL,
+      GTK_STOCK_OK,
+      GTK_RESPONSE_OK,
+      NULL);
+      
+    gtk_dialog_set_default_response (GTK_DIALOG (wizard_window), GTK_RESPONSE_OK);
+      
+    /*gtk_window_set_resizable (GTK_WINDOW(wizard_window), FALSE);*/
+    /*gtk_window_set_position(GTK_WINDOW(wizard_window), GTK_WIN_POS_CENTER);*/
+    /*gtk_widget_realize (wizard_window);*/
 
     g_strlcpy (title, "Tilda", sizeof (title));
     sprintf (title, "%s %i Config", title, tw->instance);
     gtk_window_set_title (GTK_WINDOW (wizard_window), title);
 
-    g_signal_connect (G_OBJECT (wizard_window), "delete_event",
-                  G_CALLBACK (exit_app), NULL);
+    /*g_signal_connect (G_OBJECT (wizard_window), "delete_event",
+                  G_CALLBACK (exit_app), NULL);*/
 
     gtk_container_set_border_width (GTK_CONTAINER (wizard_window), 10);
 
     table = gtk_table_new (3, 3, FALSE);
     gtk_table_set_row_spacings (GTK_TABLE (table), 10);
     gtk_table_set_col_spacings (GTK_TABLE (table), 10);
-
-    gtk_container_add (GTK_CONTAINER (wizard_window), table);
-
+    
+    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (wizard_window)->vbox), GTK_WIDGET (table), TRUE, TRUE, 7);
+    
     /* Create a new notebook, place the position of the tabs */
     notebook = gtk_notebook_new ();
     gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
@@ -844,30 +949,20 @@ int wizard (int argc, char **argv, tilda_window *tw, tilda_term *tt)
 
     gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 0);
 
-    button = gtk_button_new_with_label ("OK");
-    g_signal_connect_swapped (G_OBJECT (button), "clicked",
-                  G_CALLBACK (ok), tw);
-    gtk_table_attach_defaults (GTK_TABLE (table), button, 0, 1, 2, 3);
-    gtk_widget_show (button);
 
-    button = gtk_button_new_with_label ("Apply");
-    g_signal_connect_swapped (G_OBJECT (button), "clicked",
-                  G_CALLBACK (apply), tw);
-    gtk_table_attach_defaults (GTK_TABLE (table), button, 1, 2, 2, 3);
-    gtk_widget_show (button);
-
-    button = gtk_button_new_with_label ("Cancel");
-    g_signal_connect_swapped (G_OBJECT (button), "clicked",
-                  G_CALLBACK (exit_app), NULL);
-    gtk_table_attach_defaults (GTK_TABLE (table), button, 2, 3, 2, 3);
-    gtk_widget_show (button);
+    g_signal_connect (G_OBJECT (wizard_window), "response",
+      G_CALLBACK (wizard_window_response_cb), tw);
 
     gtk_widget_show (notebook);
     gtk_widget_show (table);
-    gtk_widget_show (wizard_window);
+    /*gtk_widget_show (wizard_window);*/
+    
+    exit_status = gtk_dialog_run (GTK_DIALOG (wizard_window)) != GTK_RESPONSE_OK;
+    
+    gtk_widget_destroy (GTK_WIDGET (wizard_window));
 
-    if (in_main)
-        gtk_main ();
+    /*if (in_main)
+        gtk_main ();*/
 
     return exit_status;
 }
